/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#ifndef __INSTRUMENT_H__
#define __INSTRUMENT_H__

#include <vector>

#include "sms/sms.h"
#undef PI

/// Modes for unit sine
enum InstrumentMode
{
    UNIT_SINE_OFF_MODE,
    UNIT_SINE_SINE_MODE,
    UNIT_SINE_SAMPLE_MODE,
    UNIT_SINE_SMS_MODE
};

/// States for unit sine
enum InstrumentState
{
    UNIT_SINE_OFF,
    UNIT_SINE_ATTACK,
    UNIT_SINE_SUSTAIN,
    UNIT_SINE_RELEASE
};

/// Basic sine wave
class Instrument
{
public:
    /// Constructor
    Instrument();

    /// Destructor
    ~Instrument();

    /// Get the sample at the specified time and sample offset
    float get_next_sample();

    /// Start playing the specified note
    void play(int note);

    /// Stop playing
    void stop();

    /// Sets the current sample to the utility's recording.  Switches
    /// to sample playback mode once the sample is loaded.
    void get_sample_from_utility();

    /// Set mode
    void set_mode(InstrumentMode mode);

    /// Run the SMS processing on the current recording, and switch to
    /// SMS mode.
    void do_sms_processing();

private:
    /// Free and clear the stored SMS data
    void free_sms_data();

    /// Synthesize the specified note
    void synthesize_note(int note);

    /// Initialize synth once to save processing power?
    void init_synth();

    /// Uninitialize synth
    void free_synth();

    /// Has synth been initialized?
    bool synth_is_initialized;

    /// The fundamental frequency of the unit generator
    float m_frequency;

    /// The current sample
    unsigned long m_current_sample;
 
    /// Current phase
    float m_phase;

    /// Current mode
    InstrumentMode m_mode;

    /// Current state
    InstrumentState m_state;

    /// Gain for scaling note transitions
    float m_gain;

    /// For sample playback mode, the recorded sample
    std::vector<float> m_sample;

    /// Buffer for currently playing SMS note
    std::vector<float> m_sms_buffer;

    /// Index into the sample
    size_t m_sample_index;

    /// Index into the sms buffer
    size_t m_sms_buffer_index;

    /// SMS analyzed frames
    std::vector<SMS_Data*> m_sms_data;

    /// Buffer for SMS synthesized sound
    std::vector<float> m_sms_sample;

    /// Header for synthesizing
    SMS_Header smsHeader;

    /// Synthesis parameters
    SMS_SynthParams synthParams;
};

#endif
