/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#include "Utility.h"
#include <cassert>
#include <cmath>
#include <cstdlib>
#include <ctime>
#include "RtAudio.h"

#include "WidgetMain.h"

// Static initializations
const float Utility::PI = std::atan(1.0)*4;
const float Utility::EPSILON = 0.0000001;
const unsigned int Utility::MAXIMUM_SAMPLE_RATE = 48000;
const std::string Utility::PROGRAM_NAME = "sequins";
const unsigned int Utility::OUTPUT_CHANNELS = 2;
unsigned int Utility::sample_rate = 0;
RtAudio* dac = NULL;
const unsigned int Utility::matrix_num_steps = 32;
const unsigned int Utility::matrix_num_note_values = 24;
const int Utility::NOTE_OFF = -1;
const int Utility::NOTE_CONTINUE = -1;
unsigned int Utility::num_tracks = 0;
int Utility::note_interval = 100;


Utility* Utility::get_utility()
{
    static Utility* instance = new Utility();
    return instance;
}

Utility::Utility()
{
    // Seed random number generator
    std::srand(std::time(NULL));
    dac = new RtAudio();
    pthread_mutex_init(&sound_mutex, NULL);
    currently_recording = false;
}

Utility::~Utility()
{
    pthread_mutex_unlock(&sound_mutex);
    pthread_mutex_destroy(&sound_mutex);

    delete dac;
}

void Utility::lock()
{
    pthread_mutex_lock(&sound_mutex);
}

void Utility::unlock()
{
    pthread_mutex_unlock(&sound_mutex);
}

float Utility::rand(float min, float max)
{
    return (min + ((max - min) * ((float)std::rand() / (float)RAND_MAX)));
}

float Utility::midi_to_frequency(int note)
{
    assert (note >=0);
    assert (note <=127);

    // Reference frequency is A440, note number 69
    const float fref=440;
    const int nref = 69;
    return (fref * std::pow(2,((float)note-(float)nref)/12.0));
}

int Utility::frequency_to_midi(float frequency)
{
    // Reference frequency is A440, note number 69
    const float fref=440;
    const int nref = 69;
    return std::floor(0.5 + ((float)nref + 12.0 * (std::log(frequency / fref) / std::log(2) )));
}

std::string Utility::midi_to_string(int note)
{
    std::string result;
    int pitch = note % 12;
    switch (pitch)
    {
    case 0:
        result = "C";
        break;
    case 1:
        result = "C#/Db";
        break;
    case 2:
        result = "D";
        break;
    case 3:
        result = "D#/Eb";
        break;
    case 4:
        result = "E";
        break;
    case 5:
        result = "F";
        break;
    case 6:
        result = "F#/Gb";
        break;
    case 7:
        result = "G";
        break;
    case 8:
        result = "G#/Ab";
        break;
    case 9:
        result = "A";
        break;
    case 10:
        result = "A#/Bb";
        break;
    case 11:
        result = "B";
        break;
    }
    return result;
}

bool Utility::is_recording()
{
    return currently_recording;
}

void Utility::start_recording()
{
    current_recording.clear();
    currently_recording = true;
}

void Utility::stop_recording()
{
    currently_recording = false;
}

void Utility::consume_sample(float sample)
{
    if(currently_recording)
    {
        current_recording.push_back(sample);
    }
}
