/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#ifndef __WIDGET_MATRIX_EDITOR__
#define __WIDGET_MATRIX_EDITOR__

#include <QWidget>
#include <vector>

class TrackData;

class WidgetMatrixEditor : public QWidget
{
    Q_OBJECT

public:
    WidgetMatrixEditor(TrackData* track_data, QWidget* parent = 0);

    /// Get the note at the specified index
    int get_note_at(unsigned int index);

    void set_beat(int beat);

    /// Fade one step
    void fade();

protected:
    void paintEvent(QPaintEvent* event);
    void mousePressEvent(QMouseEvent* event);
    void mouseMoveEvent(QMouseEvent* event);
    void mouseReleaseEvent(QMouseEvent* event);

private:

    /// y-coordinate for drawing vertical lines
    int y_line_coord(unsigned int index);
    /// x-coordinate for drawing horizontal lines
    int x_line_coord(unsigned int index);

    /// Get note from point
    int get_note_from_point(const QPoint &pos);

    /// Get subdivision from point
    int get_subdivision_from_point(const QPoint &pos);
    
    /// Get bounding box for a specific note / subdivision value
    QRect get_note_bounding_box(unsigned int subdivision, unsigned int note);

    /// Track data
    TrackData* m_track_data;

    /// Current beat, for drawing purposes
    int m_current_beat;

    /// Fade value, for drawing
    int m_fade_value;
};

#endif
