/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#ifndef __WIDGET_TRACK__
#define __WIDGET_TRACK__

#include <QWidget>

class TrackData;
class QCloseEvent;
class Instrument;
class WidgetMatrixEditor;
class QPushButton;

/// QT Widget for displaying a single track.  This is responsible for
/// handling all the data for a single track.
class WidgetTrack : public QWidget
{
    Q_OBJECT

public:
    ~WidgetTrack();
    WidgetTrack(QWidget* parent = 0);

    /// Start playing the specified timestep
    void play(unsigned int timestep);
    
    /// Get the next sample for this track
    float get_next_sample();

public slots:
    void play_beat(unsigned int beat);
    void stop_playing();

    /// Mute button callback
    void mute_slot();

    /// Record button callback
    void record_slot();

    /// Fade timeout signal
    void fade_slot();

protected:
    void closeEvent(QCloseEvent*);

private:
    /// Matrix editor
    WidgetMatrixEditor* matrix_editor;

    /// Data for the track
    TrackData* m_track_data;

    /// The instrument
    Instrument* m_instrument;

    QPushButton* mute_button;
    QPushButton* record_button;

    bool m_is_muted;
    bool m_is_recording;
};

#endif
