/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#include <QWidget>
#include <QVBoxLayout>
#include <QScrollArea>
#include <QLabel>
#include <QPainter>
#include <QLayoutItem>
#include <QTimer>

#include "Utility.h"
#include "WidgetTrack.h"
#include "WidgetTrackArea.h"

WidgetTrackArea::WidgetTrackArea(QWidget* parent)
    : QWidget(parent)
{
    layout = new QVBoxLayout();
    setLayout(layout);

    setBackgroundRole(QPalette::Shadow);

    fade_timer = new QTimer(this);
    fade_timer->start(10);
}

float WidgetTrackArea::get_next_sample()
{
    float result = 0.0;
    int num_tracks = layout->count();
    if (num_tracks > 0)
    {
        for (int i = 0; i < num_tracks; ++i)
        {
            result += ((WidgetTrack*)layout->itemAt(i)->widget())->get_next_sample();
        }
        result /= num_tracks;
    }
    return result;
}

void WidgetTrackArea::add_track()
{
    Utility::get_utility()->lock();

    WidgetTrack* new_track = new WidgetTrack(this);
    m_tracks.push_back(new_track);
    layout->addWidget(new_track);

    // Connect signals
    connect(this, SIGNAL(play_signal(unsigned int)), new_track, SLOT(play_beat(unsigned int)));
    connect(this, SIGNAL(stop_signal()), new_track, SLOT(stop_playing()));
    connect(fade_timer, SIGNAL(timeout()), new_track, SLOT(fade_slot()));

    Utility::get_utility()->unlock();
}

void WidgetTrackArea::play_beat(unsigned int beat)
{
    emit play_signal(beat);
}

void WidgetTrackArea::stop_playing()
{
    emit stop_signal();
}

void WidgetTrackArea::fade_timeout_slot()
{
    emit fade_signal();
}
