/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#include <QApplication>
#include <QFont>
#include <QPushButton>
#include <QWidget>
#include <QHBoxLayout>
#include <QSpinBox>
#include <QLabel>
#include <QFileDialog>
#include <QTimer>

#include "WidgetTransport.h"
#include "Utility.h"
#include "SongData.h"
#include "RtAudio.h"

WidgetTransport::~WidgetTransport()
{
    delete m_song_data;
}

WidgetTransport::WidgetTransport(QWidget* parent)
    : QWidget(parent)
{
    const int default_tempo = 60;

    m_song_data = new SongData();

    QPushButton* play_button = new QPushButton(tr("Play"), this);
    play_button->setMaximumSize(64,48);
    play_button->setMinimumSize(64,48);

    QPushButton* stop_button = new QPushButton(tr("Stop"), this);
    stop_button->setMaximumSize(64,48);
    stop_button->setMinimumSize(64,48);

    QLabel* tempo_label = new QLabel(tr("Tempo"), this);
    QSpinBox* tempo_spinbox = new QSpinBox(this);
    tempo_spinbox->setRange(1,800);
    tempo_spinbox->setValue(default_tempo);
    QVBoxLayout* tempo_layout = new QVBoxLayout();
    tempo_layout->addWidget(tempo_label);
    tempo_layout->addWidget(tempo_spinbox);

    QPushButton* new_track_button = new QPushButton(tr("New Track"), this);
    new_track_button->setMaximumSize(72,48);
    new_track_button->setMinimumSize(72,48);

    QPushButton* quit_button = new QPushButton(tr("Quit"), this);
    quit_button->setMaximumSize(64,48);
    quit_button->setMinimumSize(64,48);
    connect(quit_button, SIGNAL(clicked()), qApp, SLOT(quit()));

    QHBoxLayout *layout = new QHBoxLayout();
    layout->addWidget(play_button);
    layout->addWidget(stop_button);
    layout->addLayout(tempo_layout);
    layout->addWidget(new_track_button);
    layout->addWidget(quit_button);
    layout->addStretch();

    setLayout(layout);

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(timer_slot()));

    int default_interval = (60*1000)/(4 * default_tempo);
    Utility::get_utility()->lock();
    timer->start(default_interval);
    Utility::note_interval = default_interval;
    Utility::get_utility()->unlock();
    
    // Connections
    connect(new_track_button, SIGNAL(clicked()), this, SLOT(new_track_button_slot()));
    connect(play_button, SIGNAL(clicked()), this, SLOT(play_button_slot()));
    connect(stop_button, SIGNAL(clicked()), this, SLOT(stop_button_slot()));
    connect(tempo_spinbox, SIGNAL(valueChanged(int)), this, SLOT(change_tempo_slot(int)));
}

void WidgetTransport::timer_slot()
{
    if(m_song_data->m_playing)
    {
        emit play_signal(m_song_data->m_current_beat);
        m_song_data->m_current_beat++;
        m_song_data->m_current_beat %= Utility::matrix_num_steps;
    }
}

void WidgetTransport::new_track_button_slot()
{
    emit new_track();
}

void WidgetTransport::play_button_slot()
{
    m_song_data->m_current_beat = 0;
    m_song_data->m_playing = true;
}

void WidgetTransport::stop_button_slot()
{
    m_song_data->m_playing = false;
    emit stop_signal();
}

void WidgetTransport::change_tempo_slot(int new_tempo)
{
    int interval = 60 * 1000 / (4 * new_tempo);
    Utility::get_utility()->lock();
    timer->setInterval(interval);
    Utility::note_interval = interval;
    Utility::get_utility()->unlock();
}
