/* 
 * Copyright (c) 2010 Michael J. Wilson
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

// Michael J. Wilson
// Music 256a Fall 2010
// Final Project

#include <algorithm>
#include <cctype>
#include <cstdlib>
#include <pthread.h>
#include <string>
#include <cstring>
#include <sstream>

#include <iostream>

#include <QApplication>

#include "RtAudio.h"
#include "Utility.h"

#include "WidgetMain.h"


//////////////////////////////////////////////////////////////////////
/// Audio callback
//////////////////////////////////////////////////////////////////////
int audio_callback(void *outputBuffer, void *inputBuffer, unsigned int nBufferFrames, double streamTime, RtAudioStreamStatus status, void *userData)
{
    unsigned int i;
    unsigned int j;
    float* obuffer = (float*) outputBuffer;
    float* ibuffer = inputBuffer != NULL ? (float*)inputBuffer : NULL;
    WidgetMain* unit_gen = (WidgetMain*) userData;
    float output_sample;
    float input_sample;

    if (status)
    {
        std::cout << "Stream over/underflow detected!" << std::endl;
    }

    Utility::get_utility()->lock();

    for (i=0; i<nBufferFrames; i++)
    {
        input_sample = (*ibuffer++);
        Utility::get_utility()->consume_sample(input_sample);

        output_sample = unit_gen->get_next_sample();
        for (j=0; j<Utility::OUTPUT_CHANNELS; j++)
        {
            *obuffer++ = output_sample;
        }
    }

    Utility::get_utility()->unlock();

    return(0);
}

//////////////////////////////////////////////////////////////////////
/// Handle main input
//////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
    bool error_occured = false;
    unsigned int buffer_frames = 512;

    //////////////////////////////////////////////////
    // Make QT objects
    QApplication app(argc, argv);
    WidgetMain main_window;

    //////////////////////////////////////////////////
    // Set up audio
    if (Utility::get_utility()->dac->getDeviceCount() < 1)
    {
        std::cout << std::endl << "No audio devices found!" << std::endl;
        error_occured = true;
    }
    if (!error_occured)
    {
        // Get the best sample rate, up to our maximum:
        Utility::sample_rate = 0;
        RtAudio::DeviceInfo info = Utility::get_utility()->dac->getDeviceInfo(Utility::get_utility()->dac->getDefaultInputDevice());
        for (std::vector<unsigned int>::iterator sr_iter = info.sampleRates.begin(); sr_iter != info.sampleRates.end(); ++sr_iter)
        {
            if (*sr_iter <= Utility::MAXIMUM_SAMPLE_RATE && *sr_iter > Utility::sample_rate)
            {
                Utility::sample_rate = *sr_iter;
            }
        }

        RtAudio::StreamParameters output_parameters;
        output_parameters.deviceId = Utility::get_utility()->dac->getDefaultOutputDevice();
        output_parameters.nChannels = Utility::OUTPUT_CHANNELS;
        output_parameters.firstChannel = 0;

        RtAudio::StreamParameters input_parameters;
        input_parameters.deviceId = Utility::get_utility()->dac->getDefaultInputDevice();
        input_parameters.nChannels = 1;
        input_parameters.firstChannel = 0;

        try
        {
            Utility::get_utility()->dac->openStream(
                &output_parameters,
                &input_parameters,
                RTAUDIO_FLOAT32,
                Utility::sample_rate,
                &buffer_frames,
                &audio_callback,
                &main_window);
            Utility::get_utility()->dac->startStream();
        }
        catch (RtError& e)
        {
            e.printMessage();
            error_occured = true;
        }
    }

    //////////////////////////////////////////////////
    // Start main loop
    if (!error_occured)
    {
        main_window.show();
        app.exec();
    }

    //////////////////////////////////////////////////
    // Cleanup
    if (Utility::get_utility()->dac->isStreamRunning())
    {
        try
        {
            Utility::get_utility()->dac->stopStream();
        }
        catch (RtError& e)
        {
            e.printMessage();
        }
    }

    if (Utility::get_utility()->dac->isStreamOpen())
    {
        Utility::get_utility()->dac->closeStream();
    }

    return (error_occured ? EXIT_FAILURE : EXIT_SUCCESS);
}
